package vc.thinker.mqtt.proxyclient.service;

import com.google.common.collect.Lists;
import org.springframework.stereotype.Service;
import vc.thinker.absctacts.mqtt.publish.TopicMqttPushService;
import vc.thinker.mqtt.proxyclient.entity.down.PlateFormMeterInfo;
import vc.thinker.mqtt.proxyclient.entity.up.MeterListPull;
import vc.thinker.mqtt.proxyclient.entity.up.ServiceExecResult;
import vc.thinker.mqtt.proxyclient.entity.up.ThirdPartyProxyReport;
import vc.thinker.mqtt.proxyclient.enums.ThirdPartyProxyTopicDefinition;
import vc.thinker.utils.CompletableFutureObserver;

import java.util.List;
import java.util.UUID;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;

/**
 * 适配4G网关方式协议适配
 *
 * @author HeTongHao
 * @since 2021/7/9 14:05
 */
@Service
public class ThirdPartyProxyClientService extends TopicMqttPushService {
    public final CompletableFutureObserver<Object> completableFutureObserver = new CompletableFutureObserver<>();

    /**
     * 响应数据
     * 基于requestId 来实现同步响应数据
     *
     * @param requestId requestId
     * @param data      数据对象
     */
    public void responseData(String requestId, Object data) {
        completableFutureObserver.onMessage(requestId, data);
    }

    /**
     * 上报数据
     *
     * @param thirdPartyProxyReport 上报数据
     */
    public void report(ThirdPartyProxyReport thirdPartyProxyReport) {
        super.push(ThirdPartyProxyTopicDefinition.report, thirdPartyProxyReport);
    }

    /**
     * 服务执行响应
     *
     * @param serviceExecResult 服务执行响应结果
     */
    public void executeResponse(ServiceExecResult serviceExecResult) {
        super.push(ThirdPartyProxyTopicDefinition.executeServiceResponse, serviceExecResult);
    }

    /**
     * 拉取仪表列表
     *
     * @param protocolCode 协议编号
     */
    public List<PlateFormMeterInfo> pullMeterList(String protocolCode) throws TimeoutException {
        String requestId = UUID.randomUUID().toString();
        super.push(ThirdPartyProxyTopicDefinition.meterListPull, MeterListPull
                .builder()
                .protocolCode(protocolCode)
                .requestId(requestId)
                .build()
        );
        CompletableFuture<Object> subscribe = completableFutureObserver.subscribe(requestId);
        try {
            return (List<PlateFormMeterInfo>) subscribe.get(3, TimeUnit.SECONDS);
        } catch (InterruptedException | ExecutionException e) {
            e.printStackTrace();
            return Lists.newArrayList();
        } catch (TimeoutException e) {
            throw e;
        }
    }
}
